@@ -71,7 +71,7 @@ impl SerializedToken {
     /// Gets a reference to value held by Static
     fn as_static_ref(&self) -> Option<&[u8; 32]> {
         match *self {
-            SerializedToken::Static(ref data) => Some(&data),
+            SerializedToken::Static(ref data) => Some(data),
             _ => None,
         }
     }
@@ -147,7 +147,7 @@ impl Token {
                 // we treat it as separate case.
                 let mut wtr: [u8; 32] = Default::default();
                 let bytes = address.as_bytes();
-                wtr[32 - bytes.len()..].copy_from_slice(&bytes);
+                wtr[32 - bytes.len()..].copy_from_slice(bytes);
                 SerializedToken::Static(wtr)
             }
         }
@@ -338,12 +338,17 @@ pub fn encode_tokens(tokens: &[Token]) -> Vec<u8> {
 
                 // Store next dynamic buffer *after* dynamic offset is calculated.
                 dynamic_data.push(data);
-                // Convert into token for easy serialization
-                let offset: Token = dynamic_offset.into();
-                // Write the offset of the dynamic data as a value of static size.
-                match offset.serialize() {
-                    SerializedToken::Static(bytes) => res.extend(&bytes),
-                    _ => panic!("Offset token is expected to be static"),
+
+                // static structs do not require offsets as they aren't actually
+                // of dynamic length
+                if !is_static_struct_array(tokens) {
+                    // Convert into token for easy serialization
+                    let offset: Token = dynamic_offset.into();
+                    // Write the offset of the dynamic data as a value of static size.
+                    match offset.serialize() {
+                        SerializedToken::Static(bytes) => res.extend(&bytes),
+                        _ => panic!("Offset token is expected to be static"),
+                    }
                 }
             }
         }
@@ -370,11 +375,11 @@ pub fn encode_call(sig: &str, tokens: &[Token]) -> Result<Vec<u8>, Error> {
     wtr.extend(&derive_method_id(sig)?);
 
     let args_count = get_args_count(sig)?;
-    if args_count != get_tokens_count(tokens) {
+    let token_count = get_tokens_count(tokens);
+    if args_count != token_count {
         return Err(Error::InvalidCallError(format!(
             "Function call contains {} arguments, but {} provided",
-            args_count,
-            tokens.len()
+            args_count, token_count
         )));
     }
 
@@ -390,12 +395,57 @@ fn get_tokens_count(tokens: &[Token]) -> usize {
     for token in tokens {
         match token {
             Token::Struct(v) => count += get_tokens_count(v),
+            // for the case of an array of structs we count that structs members
+            // that is what we'll see in the function header
+            Token::Dynamic(d) => {
+                if is_struct_array(d) && !d.is_empty() {
+                    count += get_tokens_count(&[d[0].clone()])
+                } else {
+                    count += 1
+                }
+            }
             _ => count += 1,
         }
     }
     count
 }
 
+/// Simple utility function to detect arrays of structs
+fn is_struct_array(input: &[Token]) -> bool {
+    // arguable null case, could go either way
+    if input.is_empty() {
+        return false;
+    }
+    for t in input {
+        match t {
+            Token::Struct(_) => {}
+            _ => return false,
+        }
+    }
+    true
+}
+
+/// Simple utility function to detect arrays of structs that are all static in size
+fn is_static_struct_array(input: &[Token]) -> bool {
+    // arguable null case, could go either way
+    if input.is_empty() {
+        return false;
+    }
+    for t in input {
+        match t {
+            Token::Struct(v) => {
+                for t in v {
+                    if let SerializedToken::Dynamic(_) = t.serialize() {
+                        return false;
+                    }
+                }
+            }
+            _ => return false,
+        }
+    }
+    true
+}
+
 /// Gets the number of arguments by parsing a function signature
 /// string.
 fn get_args_count(sig: &str) -> Result<usize, Error> {
@@ -409,8 +459,10 @@ fn get_args_count(sig: &str) -> Result<usize, Error> {
     let args = sig.split(|ch| ch == '(' || ch == ')');
     let mut num_args = 0;
     for substring in args {
-        // leading or trailing ,'s
+        // leading or trailing ,'s or []
         let substring = substring.trim_matches(',');
+        let substring = substring.trim_matches(']');
+        let substring = substring.trim_matches('[');
         if !substring.is_empty() {
             num_args += substring.split(',').count();
         }
@@ -423,6 +475,7 @@ fn get_args_count(sig: &str) -> Result<usize, Error> {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::utils::hex_str_to_bytes;
 
     #[test]
     fn derive_event_signature() {
@@ -472,7 +525,7 @@ mod tests {
     }
 
     #[test]
-    fn derive_update_valset() {
+    fn derive_complex_signatures() {
         use utils::bytes_to_hex_str;
         assert_eq!(
             bytes_to_hex_str(&derive_method_id("dummyUpdateValset(address[])").unwrap()),
@@ -482,6 +535,8 @@ mod tests {
             bytes_to_hex_str(&derive_method_id("dummyUpdateValset(address[],uint256[])").unwrap()),
             "711ca6ac"
         );
+        assert_eq!(bytes_to_hex_str(&derive_method_id("updateValset((address[],uint256[],uint256,uint256,address),(address[],uint256[],uint256,uint256,address),(uint8,bytes32,bytes32)[])").unwrap()), "aca6b1c1");
+        assert_eq!(bytes_to_hex_str(&derive_method_id("submitLogicCall((address[],uint256[],uint256,uint256,address),(uint8,bytes32,bytes32)[],(uint256[],address[],uint256[],address[],address,bytes,uint256,bytes32,uint256))").unwrap()), "6941db93");
     }
 
     #[test]
@@ -575,7 +630,7 @@ mod tests {
         assert_eq!(
             result[..]
                 .chunks(32)
-                .map(|c| bytes_to_hex_str(&c))
+                .map(|c| bytes_to_hex_str(c))
                 .collect::<Vec<String>>(),
             vec![
                 "0000000000000000000000000000000000000000000000000000000000000123".to_owned(),
@@ -604,7 +659,7 @@ mod tests {
         assert_eq!(
             result[..]
                 .chunks(32)
-                .map(|c| bytes_to_hex_str(&c))
+                .map(|c| bytes_to_hex_str(c))
                 .collect::<Vec<String>>(),
             vec![
                 "0000000000000000000000000000000000000000000000000000000000000123".to_owned(),
@@ -631,7 +686,7 @@ mod tests {
         assert_eq!(
             result[..]
                 .chunks(32)
-                .map(|c| bytes_to_hex_str(&c))
+                .map(|c| bytes_to_hex_str(c))
                 .collect::<Vec<String>>(),
             vec!["00000000000000000000000000000000000000000000000000000000deadbeef".to_owned(),]
         );
@@ -645,7 +700,7 @@ mod tests {
         assert_eq!(
             result[..]
                 .chunks(32)
-                .map(|c| bytes_to_hex_str(&c))
+                .map(|c| bytes_to_hex_str(c))
                 .collect::<Vec<String>>(),
             vec![
                 "0000000000000000000000000000000000000000000000000000000000000040".to_owned(),
@@ -693,6 +748,34 @@ mod tests {
         assert_eq!(encoded_hash, bytes_to_hex_str(&get_hash(&result)))
     }
 
+    #[test]
+    fn encode_function_with_only_struct_arg() {
+        let correct = hex_str_to_bytes(
+            "0x414bf389000000000000000000000000c783df8a850f42e7f7e57013759c285caa701eb6000000000000000000000000c783df8a850f42e7f7e57013759c285caa701eb600000000000000000000000000000000000000000000000000000000000001f4000000000000000000000000c783df8a850f42e7f7e57013759c285caa701eb600000000000000000000000000000000000000000000000000000000000186a000000000000000000000000000000000000000000000000000000000000186a000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000",
+        )
+        .unwrap();
+
+        let address: Address = "0xc783df8a850f42e7F7e57013759C285caa701eB6"
+            .parse()
+            .unwrap();
+
+        let tokens: Vec<Token> = vec![
+            address.into(),
+            address.into(),
+            500u16.into(),
+            address.into(),
+            100_000u32.into(),
+            100_000u32.into(),
+            0u8.into(),
+            0u8.into(),
+        ];
+        let tokens = [Token::Struct(tokens)];
+        let sig =
+            "exactInputSingle((address,address,uint24,address,uint256,uint256,uint256,uint160))";
+        let payload = encode_call(sig, &tokens).unwrap();
+        assert_eq!(correct, payload);
+    }
+
     #[test]
     /// This test encodes an abiV2 function call, specifically one
     /// with a nontrivial struct in the header
@@ -704,6 +787,18 @@ mod tests {
         assert_eq!(encoded_method_id, format!("0x{}", bytes_to_hex_str(&res)));
     }
 
+    #[test]
+    /// This test encodes an abiV2 function call, specifically one
+    /// with a nontrivial struct in the header
+    fn encode_uniswap_header() {
+        use crate::utils::bytes_to_hex_str;
+        let signature =
+            "exactInputSingle((address,address,uint24,address,uint256,uint256,uint256,uint160))";
+        let encoded_method_id = "0x414bf389";
+        let res = derive_method_id(signature).unwrap();
+        assert_eq!(encoded_method_id, format!("0x{}", bytes_to_hex_str(&res)));
+    }
+
     #[test]
     fn test_args_count() {
         let test_signatures = [
@@ -711,6 +806,8 @@ mod tests {
             ("testCall(uint256,uint256,uint256)", 3),
             ("updateValset((address[],uint256[],uint256,uint256,address),(address[],uint256[],uint256,uint256,address),uint8[],bytes32[],bytes32[])", 13),
         ("submitLogicCall(address[],uint256[],uint256,uint8[],bytes32[],bytes32[],(uint256[],address[],uint256[],address[],address,bytes,uint256,bytes32,uint256))", 15),
+        ("updateValset((address[],uint256[],uint256,uint256,address),(address[],uint256[],uint256,uint256,address),(uint8[],bytes32[],bytes32[]))", 13),
+        ("updateValset((address[],uint256[],uint256,uint256,address),(address[],uint256[],uint256,uint256,address),(uint8,bytes32,bytes32)[])", 13)
         ];
         for (sig, count) in test_signatures.iter() {
             assert_eq!(get_args_count(sig).unwrap(), *count);
