@@ -19,13 +19,15 @@ use Error;
 /// # Example
 ///
 /// ```rust
-/// # use clarity::PrivateKey;
+/// use clarity::PrivateKey;
+/// use clarity::utils::get_ethereum_msg_hash;
 /// let private_key : PrivateKey = "0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f1e".parse().unwrap();
 /// let signature = private_key.sign_ethereum_msg("Hello, world!".as_bytes());
 /// // on the other side verifying the signature
-/// let hash = get_ethereum_msg_hash("Hello, world!".as_bytes())
-/// assert_eq(signature.recover(&hash).unwrap(), private_key.to_public_key().unwrap())
+/// let hash = get_ethereum_msg_hash("Hello, world!".as_bytes());
+/// assert_eq!(signature.recover(&hash).unwrap(), private_key.to_public_key().unwrap());
 /// ```
+
 pub fn get_ethereum_msg_hash(data: &[u8]) -> Vec<u8> {
     let digest = Keccak256::digest(data);
     let salt_string = ETHEREUM_SALT.to_string();
@@ -39,14 +41,14 @@ pub fn get_ethereum_msg_hash(data: &[u8]) -> Vec<u8> {
 pub fn hex_str_to_bytes(s: &str) -> Result<Vec<u8>, Error> {
     let s = match s.strip_prefix("0x") {
         Some(s) => s,
-        None => &s,
+        None => s,
     };
     let bytes = s
         .as_bytes()
         .chunks(2)
         .map::<Result<u8, Error>, _>(|ch| {
-            let str = str::from_utf8(&ch)?;
-            let byte = u8::from_str_radix(&str, 16)?;
+            let str = str::from_utf8(ch)?;
+            let byte = u8::from_str_radix(str, 16)?;
 
             Ok(byte)
         })
@@ -55,6 +57,23 @@ pub fn hex_str_to_bytes(s: &str) -> Result<Vec<u8>, Error> {
     Ok(bytes)
 }
 
+/// Function used for debug printing hex dumps
+/// of ethereum events with each uint256 on a new
+/// line
+pub fn debug_print_data(input: &[u8]) -> String {
+    let mut out = String::new();
+    let count = input.len() / 32;
+    out += "data hex dump\n";
+    for i in 0..count {
+        out += &format!(
+            "0x{}\n",
+            bytes_to_hex_str(&input[(i * 32)..((i * 32) + 32)])
+        )
+    }
+    out += "end hex dump\n";
+    out
+}
+
 /// This function displays a uint256 as an Ethereum address
 /// which requires specific formatting, mostly useful for logging
 /// and to avoid trying convert the Uint256 into an address
